#  - filelist.py - 
# This file is part of mp3togo

# Convert audio files to play on a mp3 player
#
# (c) Simeon Veldstra 2004, 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""Create a list of files for processing."""

import mp3togo.conf as setup

if setup.THREADED == True:
	import threading
else:
	import dummy_threading as threading

import sys, os



class FileList:
	"""A list of verified music files."""

	def __init__(self):
		self._list = []
		self._i = 0
		self._lock = threading.Lock()
		self._running = False

	def run(self):
		"""Must call run after loading the list
		and before starting the processing."""
		self._lock.acquire()
		self._running = True

	def running(self):
		return self._running

	def tuple(self):
		return tuple(self._list)

	def addfiles(self, names, block=True):
		if not self._lock.acquire(block) and not block:
			return False

		if type(names) not in (type([]), type(())):
			names = (names,)

		for name in names:
			try:
				name = setup.checkfile(name)
			except:
				continue
				
			if name not in self._list:
				self._list.append(name)

		self._lock.release()
		return True

	def addplaylist(self, lst, block=True):
		if not os.path.exists(lst):
			raise setup.ErrorNoFile

		if not self._lock.acquire(block) and not block:
			return False

		try:
			pl = file(lst)
			for line in pl:
				if line.startswith('/'):
					try:
						name = setup.checkfile(line)
					except:
						continue
					self._list.append(name)
				elif line.startswith("File") and line.find('=') > -1:
					name = line.strip().split('=', 1)[1]
					name = setup.checkfile(name)
					self._list.append(name)
		finally:
			self._lock.release()
		return True

	def addXMMS(self, session=0, block=True):
		try:
			import xmms.control
		except ImportError:
			raise setup.ErrorNoXMMSControl
		if not xmms.control.is_running:
			raise setup.ErrorXMMSNotRunning
		res = []
		for i in range(xmms.control.get_playlist_length(session)):
			res.append(xmms.control.get_playlist_file(i))

		# locks self in addfiles
		self.addfiles(res, block)

	def removefile(self, name, block=True):
		if not self._lock.acquire(block) and not block:
			return False

		while name in self._list:
			del self._list[self._list.index(name)]

		self._lock.release()
		return True

	def __iter__(self):
		if not self._lock.locked():
			raise setup.ErrorUnlocked

		for self._i in range(len(self._list)):
			yield self._list[self._i]

	def cur_file(self):
		return self._i

	def __len__(self):
		return len(self._list)

	def __contains__(self, name):
		return name in self._list


